
// Project: webCDwriter - Network CD Writing
// Author: Jrg Haeger

import java.io.*;
import javax.swing.tree.*;

class DirTreeNode implements MutableTreeNode
{
	DirTreeNode(String aName)
	{
		parent = null;
		name = aName;
		file = null;
		childs = new TreeNodesList(10);
		expanded = true;
		isDir = true;
		childsAdded = new TreeNodesList(10);
		childsRemoved = new TreeNodesList(10);
	}

	DirTreeNode(File aFile)
	{
		parent = null;
		name = aFile.getName();
		file = aFile;
		expanded = false;
		isDir = file.isDirectory();
		childsAdded = new TreeNodesList(10);
		childsRemoved = new TreeNodesList(10);
//		System.out.println("new DirTreeNode(" + aFile + ")");
	}

	public java.util.Enumeration children()
	{
		return null;
	}

	public static int compare(TreeNode a, TreeNode b)
	{
		String aStr = a.toString().toLowerCase();
		String bStr = b.toString().toLowerCase();

		if (a.isLeaf())
			if (b.isLeaf())
				return aStr.compareTo(bStr);
			else
				return 1;
		else
			if (b.isLeaf())
				return -1;
			else
				return aStr.compareTo(bStr);
	}

	public boolean getAllowsChildren()
	{
		return !isLeaf();
	}

	public TreeNode getChildAt(int childIndex)
	{
		expand();
		return childs.getNode(childIndex);
	}

	public int getChildCount()
	{
		expand();
		return childs.getNumOfNodes();
	}

	public File getFile()
	{
		return file;
	}

	public int getIndex(TreeNode node)
	{
		expand();
		int i = 0;
		while (i < childs.getNumOfNodes() && childs.getNode(i) != node)
			i++;
		if (i < childs.getNumOfNodes())
			return i;
		else
			return -1;
	}

	public TreeNode getParent()
	{
		return parent;
	}

	private void expand()
	{
		if (expanded)
			return;

//		System.out.println("expand(): " + name);
		File files[] = ((File2)file).listFiles();
		int filesNum;
		if (files == null)
			filesNum = 0;
		else
			filesNum = files.length;

		childs = new TreeNodesList(filesNum + 10);
		expanded = true;

		for (int i = 0; i < filesNum; i++)
		{
			File f = files[i];
			if (!Mode.audioCD()
				|| f.isDirectory()
				|| f.getName().toLowerCase()
					.endsWith(".wav")
				|| (CDcreator.mp3Support
					&&  f.getName().toLowerCase()
						.endsWith(".mp3")))
				insert2(new DirTreeNode(f));
		}
	}

	public void insert(MutableTreeNode child)
	{
		insert2(child);
		file = null;
		childsAdded.append(child);
	}

	public void insert(MutableTreeNode child, int index)
	{
		insert2(child, index);
		file = null;
		childsAdded.append(child);
	}

	private void insert2(MutableTreeNode child)
	{
		expand();
		int i;
		for (i = 0; i < childs.getNumOfNodes()
			&& compare(childs.getNode(i), child) < 0; i++);
		insert2(child, i);
	}

	private void insert2(MutableTreeNode child, int index)
	{
		expand();
		childs.insert(child, index);
		child.setParent(this);
	}

	public boolean isChanged()
	{
		return childsAdded.getNumOfNodes() > 0
			|| childsRemoved.getNumOfNodes() > 0
			|| file instanceof ServerFile;
	}

	public boolean isExpanded()
	{
		return expanded;
	}

	public boolean isLeaf()
	{
//		return file != null && !file.isDirectory();
		return !isDir;
	}

	public void remove(int index)
	{
		if (index < 0)
		{
			System.out.println("DirTreeNode.remove: index = "
				+ index);
			return;
		}

		childsRemoved.append(childs.getNode(index));
		((MutableTreeNode)childs.getNode(index)).setParent(null);
		childs.remove(index);
		file = null;
	}

	public void remove(MutableTreeNode node)
	{
		remove(getIndex(node));
	}

	public void removeFromParent()
	{
		((MutableTreeNode)getParent()).remove(this);
	}

	public void resetExpanded()
	{
		expanded = false;
	}

	public void setParent(MutableTreeNode newParent)
	{
		parent = newParent;
	}

	public void setUserObject(Object object)
	{
//		System.out.println(
//			"DirTreeNode::setUserObject: " + object);
		name = (String)object;
	}

	public String toString()
	{
		return name;
	}

	private TreeNode parent;
	private String name;
	private File file;
	private boolean expanded, isDir;
	private TreeNodesList childs, childsAdded, childsRemoved;

	class TreeNodesList
	{
		public TreeNodesList(int initialSize)
		{
			nodes = new TreeNode[initialSize];
			numOfNodes = 0;
		}

		public void append(TreeNode node)
		{
			insert(node, numOfNodes);
		}

		public TreeNode getNode(int index)
		{
			return nodes[index];
		}

		public int getNumOfNodes()
		{
			return numOfNodes;
		}

		public void insert(TreeNode node, int index)
		{
			if (index > numOfNodes)
				return;

			if (numOfNodes == nodes.length)
			{
				TreeNode[] old = nodes;
				nodes = new TreeNode[numOfNodes + 100];
				for (int i = 0; i < numOfNodes; i++)
					nodes[i] = old[i];
			}

			for (int i = numOfNodes; i > index; i--)
				nodes[i] = nodes[i-1];
			nodes[index] = node;
			numOfNodes++;
		}

		public void remove(int index)
		{
			if (index < 0)
				return;

			for (int i = index; i < numOfNodes - 1; i++)
				nodes[i] = nodes[i+1];
			numOfNodes--;
		}

		private TreeNode nodes[];
		private int numOfNodes;
	}
}
